@@ -87,7 +87,7 @@ sub run {
         }
 
         next if /^\s*\#/;
-        if (m/^\s+-?(dh_\S+)/) {
+        if (m/^\s+[@+-]?(dh_\S+)/) {
             my $dhcommand = $1;
             $build_systems{'debhelper'} = 1
               if not exists($build_systems{'dh'});
@@ -131,7 +131,7 @@ sub run {
             }
             $seencommand = 1;
             $needbuilddepends = 1;
-        } elsif (m,^\s+dh\s+,) {
+        } elsif (m,^\s+[@+-]?dh\s+,) {
             $build_systems{'dh'} = 1;
             delete($build_systems{'debhelper'});
             $seen_dh = 1;
