@@ -191,8 +191,6 @@ my $BUILD_PATH_REGEX
   = Lintian::Data->new('files/build-path-regex',qr/~~~~~/,
     sub { return  qr/$_[0]/xsm;});
 
-my @ALLOWED_USES_DPKG_DATABASE = qw(base-files dpkg lintian);
-
 sub _tag_build_tree_path {
     my ($path, $msg) = @_;
     foreach my $buildpath ($BUILD_PATH_REGEX->all) {
@@ -299,9 +297,9 @@ my $OBSOLETE_PATHS = Lintian::Data->new(
     });
 
 sub run {
-    my ($pkg, $type, $info, $proc) = @_;
+    my ($pkg, $type, $info, $proc, $group) = @_;
     my ($is_python, $is_perl, $has_binary_perl_file, $has_public_executable,
-        $has_public_shared_library);
+        $has_public_shared_library, $build_path);
     my @nonbinary_perl_files_in_lib;
     my %linked_against_libvga;
     my @devhelp;
@@ -320,6 +318,11 @@ sub run {
     my $multiarch_dir = $MULTIARCH_DIRS->value($arch);
     my $ppkg = quotemeta($pkg);
 
+    my $buildinfo = $group->get_buildinfo_processable;
+    if ($buildinfo) {
+        $build_path = $buildinfo->info->field('build-path', '');
+    }
+
     my %header_dirs = ('usr/include/' => 1);
     foreach my $x ($MULTIARCH_DIRS->all) {
         my $dir = $MULTIARCH_DIRS->value($x);
@@ -347,8 +350,6 @@ sub run {
     # Check if package is empty
     my $is_dummy = $info->is_pkg_class('any-meta');
 
-    my $has_sensible_utils = $info->relation('all')->implies('sensible-utils');
-
     # read data from objdump-info file
     foreach my $file (sort keys %{$info->objdump_info}) {
         my $objdump = $info->objdump_info->{$file};
@@ -420,13 +421,26 @@ sub run {
         my $owner = $file->owner . '/' . $file->group;
         my $operm = $file->operm;
         my $link = $file->link;
+        my $finfo = $file->file_info;
 
-        $arch_dep_files = 1 if $fname !~ m,^usr/share/,o && $fname ne 'usr/';
+        $arch_dep_files = 1
+          if not $file->is_dir
+          and $fname !~ m,^usr/share/,o
+          and $finfo
+          and $finfo !~ m/\bASCII text\b/;
+
+        if ($file->dirname =~ m,^(?:usr)?/lib/([^/]+)/$,) {
+            $arch_dep_files = 1 if $TRIPLETS->known($1 // '');
+        }
 
         if (exists($PATH_DIRECTORIES{$file->dirname})) {
             $has_public_executable = 1;
             tag 'file-name-in-PATH-is-not-ASCII', $file
               if $file->basename !~ m{\A [[:ascii:]]++ \Z}xsm;
+            tag 'zero-byte-executable-in-path', $file
+              if $file->is_regular_file
+              and $file->is_executable
+              and $file->size == 0;
         } elsif (!is_string_utf8_encoded($fname)) {
             tag 'file-name-is-not-valid-UTF-8', $file;
         }
@@ -437,6 +451,12 @@ sub run {
             tag $tag, $file if $fname =~ m/$regex/;
         }
 
+        tag 'package-contains-usr-unmerged-pathnames', $file
+          if $fname =~ m,^(?:bin|sbin|lib.*)/.+$,
+          and not $file->is_symlink
+          and not $file->is_dir
+          and ($link // '') !~ m,^usr/$fname$,;
+
         if ($file->is_hardlink) {
             my $link_target_dir = $link;
             $link_target_dir =~ s,[^/]*$,,;
@@ -1498,37 +1518,35 @@ sub run {
                 tag 'nfs-temporary-file-in-package', $file;
             }
 
-            # ---------------- using sensible-utils w/o dependency
-            tag 'missing-depends-on-sensible-utils', $file
-              if not $has_sensible_utils
-              and $fname !~ m,^usr/share/(?:doc|locale)/,
-              and $source_pkg ne 'lintian'
-              and detect_sensible_utils($file);
+            # ---------------- contents checks
+            my %checks
+              = get_checks_for_file($info, $file, $source_pkg, $build_path);
 
-            # ---------------- using dpkg internals
-            if ($fname !~ m,^usr/share/(?:doc|locale)/,
-                and none { $_ eq $source_pkg } @ALLOWED_USES_DPKG_DATABASE) {
+            if (%checks) {
                 my $strings = slurp_entire_file($info->strings($file));
 
-                # If we have strings(1) output (eg. we are an ELF
-                # binary) then prefer that.
-                if ($strings) {
-                    tag 'uses-dpkg-database-directly', $file
-                      if $strings =~ m,^/var/lib/dpkg,m;
-                } else {
-                    my $fd = $file->open(':raw');
-                    my $sfd = Lintian::SlidingWindow->new($fd);
-                    while (my $block = $sfd->readwindow) {
-                        next unless $block =~ m,/var/lib/dpkg,;
-                        tag 'uses-dpkg-database-directly', $file;
-                        last;
+                foreach my $tag (sort keys %checks) {
+                    my $regex = $checks{$tag};
+
+                    # If we have strings(1) output (eg. we are an ELF
+                    # binary) then prefer that.
+                    if ($strings) {
+                        tag $tag, $file if $strings =~ m,^\Q$regex\E,m;
+                    } else {
+                        my $fd = $file->open(':raw');
+                        my $sfd = Lintian::SlidingWindow->new($fd);
+                        while (my $block = $sfd->readwindow) {
+                            next unless $block =~ $regex;
+                            tag $tag, $file;
+                            last;
+                        }
+                        close($fd);
                     }
-                    close($fd);
                 }
             }
 
             # ---------------- documentation files
-            unless($fname =~ m,^etc/, or $fname =~ m,^usr/share/doc/,) {
+            unless($fname =~ m,^etc/, or $fname =~m,^usr/share/(?:doc|help)/,){
                 foreach my $taboo ($DOCUMENTATION_FILE_REGEX->all) {
                     my $regex = $DOCUMENTATION_FILE_REGEX->value($taboo);
                     if($file->basename =~ m{$regex}xi) {
@@ -2031,14 +2049,13 @@ sub run {
         my $file = $info->control_index_resolved_path($1);
         next if not $file or not $file->is_open_ok;
 
-        tag 'missing-depends-on-sensible-utils', $file
-          if not $has_sensible_utils and detect_sensible_utils($file);
-
+        my %checks
+          = get_checks_for_file($info, $file, $pkg_section, $build_path);
         my $fd2 = $file->open;
         while (<$fd2>) {
-            tag 'uses-dpkg-database-directly', $file, "(line $.)"
-              if m,/var/lib/dpkg,
-              and none { $_ eq $source_pkg } @ALLOWED_USES_DPKG_DATABASE;
+            foreach my $tag (sort keys %checks) {
+                tag $tag, $file, "(line $.)" if $_ =~ $checks{$tag};
+            }
         }
         close($fd2);
     }
@@ -2343,18 +2360,28 @@ sub detect_privacy_breach {
     return;
 }
 
-sub detect_sensible_utils {
-    my ($file) = @_;
+sub get_checks_for_file {
+    my ($info, $file, $source_pkg, $build_path) = @_;
+    my %checks;
 
-    my $fd = $file->open(':raw');
-    my $sfd = Lintian::SlidingWindow->new($fd, sub { $_= lc($_); }, BLOCKSIZE);
+    return %checks if $source_pkg eq 'lintian';
 
-    while (my $block = $sfd->readwindow) {
-        return 1
-          if $block=~ m/(?:select-editor|sensible-(?:browser|editor|pager))\b/;
-    }
-    close($fd);
-    return;
+    $checks{'missing-depends-on-sensible-utils'}
+      = '(?:select-editor|sensible-(?:browser|editor|pager))\b'
+      if $file !~ m,^usr/share/(?:doc|locale)/,
+      and not $info->relation('all')->implies('sensible-utils');
+
+    $checks{'uses-dpkg-database-directly'} = '/var/lib/dpkg'
+      if $file !~ m,^usr/share/(?:doc|locale)/,
+      and $file->basename !~ m/^README(?:\..*)?$/
+      and $file->basename !~ m/^changelog(?:\..*)?$/i
+      and $file->basename !~ m/\.(?:html|txt)$/i
+      and $info->field('section', '') ne 'debian-installer'
+      and none { $_ eq $source_pkg } qw(base-files dpkg lintian);
+
+    $checks{'file-references-package-build-path'} = $build_path if $build_path;
+
+    return %checks;
 }
 
 1;
