@@ -459,6 +459,7 @@ sub run {
         my @symbols_depends;
         my $dep_templates = 0;
         my %meta_info_seen;
+        my $build_depends_seen = 0;
         my $warned = 0;
         my $symbol_count = 0;
 
@@ -535,6 +536,7 @@ sub run {
                   unless defined $soname and $symbol_count == 0;
 
                 $meta_info_seen{$1} = 1;
+                $build_depends_seen = 1 if $1 eq 'Build-Depends-Package';
             } elsif (m/^\s+(\S+)\s(\S+)(?:\s(\S+(?:\s\S+)?))?$/) {
                 # Symbol definition
 
@@ -603,7 +605,7 @@ sub run {
               unless $symbols_control_used{$shlib_name};
         }
         tag 'symbols-file-missing-build-depends-package-field'
-          unless exists $meta_info_seen{'Build-Depends-Package'};
+          unless $build_depends_seen;
 
         # Check that all of the packages listed as dependencies in the symbols
         # file are satisfied by the current package or its Provides.
