@@ -36,7 +36,7 @@ use Lintian::Util qw(internal_error lstrip rstrip);
 
 use Lintian::Data;
 
-# Init script that do not need a service file
+# Init scripts that do not need a service file
 my $INIT_WHITELIST = Lintian::Data->new('systemd/init-whitelist');
 
 # Known security flags
@@ -118,9 +118,9 @@ sub check_init_script {
     my $fh = $file->open;
     while (<$fh>) {
         lstrip;
-        if ($. == 1 and m{\A [#]! \s*/lib/init/init-d-script}xsm) {
-            $lsb_source_seen = 1;
-        }
+        $lsb_source_seen = 1
+          if $. == 1
+          and m{\A [#]! \s* (?:/usr/bin/env)? \s* /lib/init/init-d-script}xsm;
         if (m,#.*Default-Start:.*S,) {
             $is_rcs_script = 1;
         }
@@ -178,7 +178,7 @@ sub get_systemd_service_names {
 
     for my $file (@{$files_ref}) {
         my $name = $file->basename;
-        $name =~ s/\.service$//;
+        $name =~ s/@?\.service$//;
         $safe_add_service->($name);
 
         my @aliases= extract_service_file_values($file, 'Install', 'Alias', 1);
@@ -223,8 +223,13 @@ sub check_systemd_service_file {
     }
 
     if (not $file->is_symlink or $file->link ne '/dev/null') {
+        #<<< no perltidy
         my @wanted_by
-          = extract_service_file_values($file, 'Install', 'WantedBy',1);
+          = extract_service_file_values($file, 'Install', 'WantedBy', 1);
+        my $is_oneshot =
+           any { /^oneshot$/ }
+           extract_service_file_values($file, 'Service', 'Type', 1);
+        #>>>
         foreach my $target (@wanted_by) {
             tag 'systemd-service-file-refers-to-unusual-wantedby-target',
               $file, $target
@@ -233,14 +238,12 @@ sub check_systemd_service_file {
         }
         tag 'systemd-service-file-missing-documentation-key', $file,
           unless extract_service_file_values($file, 'Unit', 'Documentation',1);
-        #<<< no perltidy - These are better formatted one per line
         tag 'systemd-service-file-missing-install-key', $file,
-          unless @wanted_by
+             unless @wanted_by
           or extract_service_file_values($file, 'Install', 'RequiredBy',1)
           or extract_service_file_values($file, 'Install', 'Also',1)
-          or any { /^oneshot$/ } extract_service_file_values($file, 'Service', 'Type')
+          or $is_oneshot
           or $file =~ m,@\.service$,;
-        #>>>
         my @pidfile = extract_service_file_values($file,'Service','PIDFile',1);
         foreach my $x (@pidfile) {
             tag 'systemd-service-file-pidfile-refers-to-var-run', $file, $x
@@ -253,7 +256,9 @@ sub check_systemd_service_file {
             last;
         }
         tag 'systemd-service-file-missing-hardening-features', $file
-          unless $seen_hardening;
+          unless $seen_hardening
+          or $is_oneshot
+          or any { 'sleep.target' eq $_ } @wanted_by;
     }
 
     return 1;
